
#include "PhantomDetectorConstruction.hh"

#include "G4Element.hh"
#include "G4Material.hh"

#include "G4VSolid.hh"
#include "G4Tubs.hh"
#include "G4Sphere.hh"
#include "G4LogicalVolume.hh"
#include "G4VPhysicalVolume.hh"
#include "G4PVPlacement.hh"
#include "G4SubtractionSolid.hh"

#include "G4SDManager.hh"
#include "G4VSensitiveDetector.hh"
#include "G4RunManager.hh"

#include "G4VisAttributes.hh"
#include "G4Colour.hh"

#include "PhantomEye.hh"

PhantomDetectorConstruction::PhantomDetectorConstruction() :
   Vacuum(0), Water(0), PMMA(0), WorldPhysical(0), PhantomPhysical(0),
   ModeratorPhysical(0), PhantomEyeSD(0), WorldVisAtt(0), PhantomVisAtt(0),
   ModeratorVisAtt(0)
{
}


PhantomDetectorConstruction::~PhantomDetectorConstruction()
{
   // Destroy Materials and Elements
   G4ElementTable * TheElementsTable = (G4ElementTable*)(
      G4Element::GetElementTable());
   TheElementsTable->clear();
   G4MaterialTable * TheMaterialsTable = (G4MaterialTable*)(
      G4Material::GetMaterialTable());
   TheMaterialsTable->clear();

   delete PhantomEyeSD;

   delete WorldVisAtt;
   delete PhantomVisAtt;
   if (ModeratorVisAtt)
      delete ModeratorVisAtt;
}


void PhantomDetectorConstruction::ConstructMaterials()
{
   G4double A;
   G4double Z;
   G4double Density;
   G4double Weight;
   G4int    NumElements;

   // Elements:
   // ---------

   // Hidrogen
   A = 1.01*g/mole;
   G4Element * Hydrogen = new G4Element ("Hydrogen","H",Z=1.0,A);

   // Oxygen
   A = 16.00*g/mole;
   G4Element * Oxygen = new G4Element("Oxygen","O",Z=8.,A);

   // Carbon
   A = 12.01*g/mole;
   G4Element * Carbon = new G4Element("Carbon","C",Z=6.0,A);

   // Materials:
   // ----------

   // Vacuum
   Density = universe_mean_density;   //from PhysicalConstants.h
   G4double Pressure    = 3.e-18*pascal;
   G4double Temperature = 2.73*kelvin;
   Vacuum = new G4Material("Vacuum", Z=1., A=1.01*g/mole, 
			   Density, kStateGas,
			   Temperature, Pressure);
   

   // Water for phantom
   Density = 1.0*g/cm3;
   Water = new G4Material("Water",Density,NumElements=2);
   Water->AddElement(Hydrogen,2);
   Water->AddElement(Oxygen,1);

   return;
}

G4VPhysicalVolume * PhantomDetectorConstruction::Construct()
{
   ConstructMaterials();

   // World Volume
   G4VSolid * WorldSolid = new G4Tubs("WorldTube",
				      0.0*mm,      // Inner radius
				      12.0*mm,     // Outer radius
				      40.0*mm,     // Half length in Z
				      0.0,         // Starting angle
				      2.0*pi);     // Ending angle
   G4LogicalVolume * WorldLogical = new G4LogicalVolume(WorldSolid,
							Vacuum,
							"WorldLogical",
							0,0,0);
   WorldPhysical = new G4PVPlacement(0, G4ThreeVector(), WorldLogical,
				     "WorldPhysical", 0, false, 0);

   // Phantom (Eye)
   G4VSolid * PhantomSolid = new G4Sphere("PhantomSolid",
					  0.0*mm,     // Inner radius
					  12.0*mm,    // Outer radius
					  0.0,        // Starting Phi angle
					  2.0*pi,     // Delta Phi angle
					  0.0,        // Starting Theta angle
					  2.0*pi);    // Delta Theta angle
				     
   G4LogicalVolume * PhantomLogical = new G4LogicalVolume(PhantomSolid,
							  Water,
							  "PhantomLogical");
   PhantomPhysical = new G4PVPlacement(0,G4ThreeVector(0.0,0.0,25.0*mm),
				       "Phantom",PhantomLogical,
				       WorldPhysical,false,0);



   // Sensitive detector
   G4SDManager * SDManager = G4SDManager::GetSDMpointer();
   
   PhantomEyeSD = new PhantomEye("/PhantomEye");
   SDManager->AddNewDetector(PhantomEyeSD);
   PhantomLogical->SetSensitiveDetector(PhantomEyeSD);

   // Visualization
   WorldVisAtt = new G4VisAttributes(G4Colour(1.0,1.0,1.0));
   WorldVisAtt->SetVisibility(false);
   WorldLogical->SetVisAttributes(WorldVisAtt);

   PhantomVisAtt = new G4VisAttributes(G4Colour(0.3,0.0,0.0));
   PhantomVisAtt->SetForceWireframe(true);
   PhantomLogical->SetVisAttributes(PhantomVisAtt);

   
   return WorldPhysical;

}






